class Provider < ActiveRecord::Base
  belongs_to :provider_status
  
  validates :name, :presence => true, :length => {:maximum => 50}, :uniqueness => true
  validates :caption, :presence => true, :length => {:maximum => 30}
  validates :url, :presence => true, :length => {:maximum => 200}, :url => {:message => I18n.t('errors.messages.url')}
  validates :description, :presence => true
  validates :demander_url, :presence => true, :length => {:maximum => 200}, :url => {:message => I18n.t('errors.messages.url')}
  validates :provider_status_id, :presence => true, :numericality => true, :existence => {:both => false}

  def supply_default
    self.provider_status_id = nil
  end
  
  def overwrite
  end
  
  def own? ad
    return false unless ad
    ad.is_a?(Admin)
  end
  
  def visible? ad
    return false unless ad
    ad.is_a?(Admin)
  end
  
  def status
    self.provider_status.status
  end
  
  def self.default_page_size
    25
  end
  
  def self.max_page_size
    100
  end
  
  def self.default_panel_size
    30
  end
  
  def self.page prm = nil
    page = prm.to_i
    page = 1 if page < 1
    page
  end
  
  def self.page_size prm = self.default_page_size
    page_size = prm.to_i
    page_size = self.max_page_size if page_size > self.max_page_size
    page_size = self.default_page_size if page_size < 1
    page_size
  end
  
  def self.list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Provider.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:order => 'providers.name'})
    Provider.find(:all, opt)
  end
  
  def self.available_list page = 1, page_size = self.default_page_size
    opt = {}
    opt.merge!(Provider.list_opt)
    opt.merge!({:limit => page_size, :offset => (page -1) * page_size}) if page_size > 0
    opt.merge!({:conditions => ['provider_statuses.token is null'], :order => 'providers.name'})
    Provider.find(:all, opt)
  end
  
  def self.list_opt
    {:include => {:provider_status => {}} }
  end
  
  def self.list_json_opt
    {:include => {:provider_status => {}} }
  end
  
  def self.show pid, ad
    opt = {}
    opt.merge!(Provider.show_opt)
    res = Provider.find(pid, opt)
    raise ActiveRecord::Forbidden unless res.visible?(ad)
    res
  end
  
  def self.edit pid, ad
    opt = {}
    opt.merge!(Provider.show_opt)
    res = Provider.find(pid, opt)
    raise ActiveRecord::Forbidden unless res.own?(ad)
    res
  end
  
  def self.show_opt
    {:include => {:provider_status => {}} }
  end
  
  def self.show_json_opt
    {:include => {:provider_status => {}} }
  end
  
  def self.store name, attr
    r = Provider.modify_object name, attr
    Provider.transaction do
      if r.new_record?
        ps = ProviderStatus.new
        ps.supply_default
        if ps.save
          r.provider_status_id = ps.id
        else
          r.provider_status_id = nil
       end
      end
      raise ActiveRecord::Rollback unless r.save
    end
    r
  end
  
  def self.import urls
    Provider.import_urls(urls) {|name, attr| Provider.store(name, attr)}
  end
  
  def export_url action
    u = self.demander_url + (self.demander_url[-1] == '/' ? '' : '/')
    u = URI.join(u, action + '.json')
    u.to_s
  end
  
end
